<?php
/*
Plugin Name: Truth
Plugin URI: http://vandercar.net/wp
Description: Generates YouVersion links for Biblical scripture references
Version: 1.7
Author: Joshua Vandercar
Author URI: http://vandercar.net
*/

/*
Developed as a project of World Mission Prayer League
This plugin is a derivative of Verse Links, authored by BelieversResource.com
*/

/**************************************
* INITIALIZE SETTINGS AND INCLUDES
**************************************/
define( 'TRUTH_VERSION', '1.7' );
define( 'TRUTH_PLUGIN_URL', plugin_dir_url( __FILE__ ) );
define( 'TRUTH_DIR', plugin_dir_path( __FILE__ ) );
define( 'TRUTH_AUTH_ALL', FALSE );

is_admin() ? require_once( plugin_dir_path( __FILE__ ) . 'wp-side-notice/class-wp-side-notice.php' ) : FALSE;

global $truth_plugin_name;
$truth_plugin_name = 'Truth';
global $truth_prefix;
$truth_prefix = 'truth_';

global $truth_options;
$truth_options = get_option( 'truth_settings' );

global $truth_bible_versions;
$truth_bible_versions = truth_define_bible_versions();
global $truth_sites;
$truth_sites = init_truth_sites();
global $truth_bookHash;
$truth_bookHash = array();
populate_book_hash();

function truth_check_plugin_update() {

	(float) TRUTH_VERSION > (float) get_option( 'truth_db_version' ) ? truth_add_wpsn_notices() : FALSE;

}
add_action( 'admin_init', 'truth_check_plugin_update' );

function truth_add_wpsn_notices() {
	$truth_wpsn = new WP_Side_Notice( 'truth' );

	$truth_wpsn_notices = array(
		'info' => array(
			'name' => 'info',
			'trigger' => TRUE,
			'time' => time() - 5,
			'dismiss' => 'undismiss',
			'content' => '<a href="http://wordpress.org/plugins/truth">Truth</a> plugin developed by <a href="http://vandercar.net/wp">UaMV</a>.',
			'style' => array( 'height' => '72px', 'color' => '#dd9701', 'icon' => 'f331' ),
			'location' => array( 'options-reading.php' ),
			),
		'support' => array(
			'name' => 'support',
			'trigger' => TRUE,
			'time' => time() - 5,
			'dismiss' => 'forever',
			'content' => 'Require assistance? Find (or offer) support in our <a href="http://wordpress.org/support/plugin/truth/">support forum</a>.',
			'style' => array( 'height' => '72px', 'color' => '#dd9701', 'icon' => 'f338' ),
			'location' => array( 'options-reading.php' ),
			),
		'give' => array(
			'name' => 'give',
			'trigger' => TRUE,
			'time' => time() - 5,
			'dismiss' => 'forever',
			'content' => 'How are we doing? Consider <a href="http://wordpress.org/support/view/plugin-reviews/truth#postform">reviewing the plugin</a> or <a href="http://vandercar.net/wp">supporting our mission</a>.',
			'style' => array( 'height' => '', 'color' => '#dd9701', 'icon' => 'f313' ),
			'location' => array( 'options-reading.php' ),
			)
		);
	
	foreach ( $truth_wpsn_notices as $notice => $args ) {
		$truth_wpsn->add( $args );
	}

	update_option( 'truth_db_version', TRUTH_VERSION );
	
}
register_activation_hook( __FILE__, 'truth_add_wpsn_notices');

/**************************************
* CHECK IF AUTHORIZATION GRANTED & CALL NOTICE IF NEEDED
**************************************/
if ( is_admin() ) {
	if ( TRUTH_AUTH_ALL ) {
		$truth_options['authorize_external_links'] = '1';
		update_option( 'truth_settings', $truth_options );
	} else {
		if ( ! isset( $truth_options['authorize_external_links'] ) ) {
			add_action( 'admin_notices', 'truth_notices' );
		} else {
			'0' == $truth_options['authorize_external_links'] ? add_action( 'admin_notices', 'truth_notices' ) : FALSE;
		}
	}
}

/**************************************
* CREATE SHORTCODE
**************************************/

function truth_shortcode( $truth_atts, $content = '' ) {
	global $truth_options;
	extract( shortcode_atts( array(
			'version' => '',
		), $truth_atts ) );

	$verse_array = parse_verse( $content );
	$bibleVersion = truth_get_bible_version_from_abbr( strtolower( $version ) );
	
	$truth_url = get_verse_url( $verse_array[0], $verse_array[1], $verse_array[2], $verse_array[3], $verse_array[4], $bibleVersion['id'] );
	if ( '' != $version && strpos( $truth_url, strtolower( $version ) ) > 0 ) {
		switch ( $truth_options['append_version'] ) {
			case 'none':
				break;
			case 'abbr':
				$content .= ', ' . strtoupper( $bibleVersion['abbr'] );
				break;
			case 'full':
				$content .= ', ' . $bibleVersion['name'];
				break;
			default:
				break;
		}
		$truth_target = $truth_options['link_in_new_tab'] ? ' target="_blank"' : '';
		return '<a href="' . $truth_url . '" class="verseLink"' . $truth_target . '>' . $content . '</a>';
	} else {
		return $content;
	}
}
add_shortcode( 'truth', 'truth_shortcode' );

/**************************************
* FIND VERSES AND CREATE LINKS
**************************************/

function link_verses( $text ) {
	
	if ( ! class_exists('WP_Front_End_Editor') || ( class_exists('WP_Front_End_Editor') && ! WP_Front_End_Editor::is_edit() ) ) {
		global $truth_options;
		/*
		global $truth_bible_versions;
		$version_regex = 'bible';
		foreach ( $truth_bible_versions as $languageGroup => $languageVersions ) {
			foreach ( $languageVersions as $versionID => $versionInfo ) {
				$version_regex .= '|' . $versionInfo['abbr'];
			}
		}
		*/

		$pattern = '/([0-3Ii]{0,3}|Song of|First|Second|Third|1st|2nd|3rd|Primero|Segundo|Tercero|Segunda|Tercera)[ ]{0,1}[\p{L}.]{1,20} \\d{1,3}(?!\\d)[:]{0,1}[\\d,]*[-]{0,1}[\\d,]*/u';
		
		preg_match_all( $pattern, $text, $matches, PREG_OFFSET_CAPTURE );

		for ( $i = count( $matches[0] ) - 1; $i >= 0 ; $i-- ) {
			$match = $matches[0][ $i ][0];
			$idx = $matches[0][ $i ][1];

			$verse_array = parse_verse( $match );
			$truth_url = get_verse_url( $verse_array[0], $verse_array[1], $verse_array[2], $verse_array[3], $verse_array[4] );

			if ( $truth_url != '' ) {
				substr( $match, 0, 1) == ' ' ? $idx++ : FALSE;  //Make sure the $idx isn't thrown off when it trims
				$match = trim( $match );
				$before = substr( $text, 0, $idx );
				$after = substr( $text, $idx + strlen( $match ), strlen( $text ) - $idx - strlen( $match ) );
				$closeAnchor = strpos( $after, '</a' );
				$openAnchor = strpos( $after, '<a' );
				// is the following needed?
				$insertLink = FALSE;
				if ( $closeAnchor === FALSE && $openAnchor === FALSE ) {
					$insertLink = TRUE;
				} elseif ( $closeAnchor !== FALSE && $openAnchor !== FALSE && $closeAnchor > $openAnchor ) {
					$insertLink = TRUE;
				}

				if ( $insertLink === TRUE ) {
					$truth_target = $truth_options['link_in_new_tab'] ? ' target="_blank"' : '';
					$text = $before . '<a href="' . $truth_url . '" class="verseLink"' . $truth_target . '>' . $match . "</a>" . $after;
				}
			}
		}
	}

	return $text;
}

$truth_disable = isset( $truth_options['disable_auto_links'] ) ? TRUE : FALSE;
if ( isset( $truth_options['authorize_external_links'] ) ) {
	if ( $truth_options['authorize_external_links'] && ! $truth_disable ) {
			add_filter( 'the_content', 'link_verses', 99 );
			add_filter( 'comment_text', 'link_verses', 99 );
			add_filter( 'widget_text', 'link_verses', 99 );
	}
}

/**************************************
* PARSE THE VERSE
**************************************/

function parse_verse( $text ) {
	global $truth_bookHash;

	$firstSplit = explode( ':', $text ); //separate chapter and verse
	$idx = strrpos( $firstSplit[0], ' ' );
	$bookPart = str_replace( '.', '', substr( $firstSplit[0] , 0, $idx ) );
	$bookPart = strtolower( trim( $bookPart ) );
	$unwanted_array = array(    'Š'=>'S', 'š'=>'s', 'Ž'=>'Z', 'ž'=>'z', 'À'=>'A', 'Á'=>'A', 'Â'=>'A', 'Ã'=>'A', 'Ä'=>'A', 'Å'=>'A', 'Æ'=>'A', 'Ç'=>'C', 'È'=>'E', 'É'=>'E',
                            'Ê'=>'E', 'Ë'=>'E', 'Ì'=>'I', 'Í'=>'I', 'Î'=>'I', 'Ï'=>'I', 'Ñ'=>'N', 'Ò'=>'O', 'Ó'=>'O', 'Ô'=>'O', 'Õ'=>'O', 'Ö'=>'O', 'Ø'=>'O', 'Ù'=>'U',
                            'Ú'=>'U', 'Û'=>'U', 'Ü'=>'U', 'Ý'=>'Y', 'Þ'=>'B', 'ß'=>'Ss', 'à'=>'a', 'á'=>'a', 'â'=>'a', 'ã'=>'a', 'ä'=>'a', 'å'=>'a', 'æ'=>'a', 'ç'=>'c',
                            'è'=>'e', 'é'=>'e', 'ê'=>'e', 'ë'=>'e', 'ì'=>'i', 'í'=>'i', 'î'=>'i', 'ï'=>'i', 'ð'=>'o', 'ñ'=>'n', 'ò'=>'o', 'ó'=>'o', 'ô'=>'o', 'õ'=>'o',
                            'ö'=>'o', 'ø'=>'o', 'ù'=>'u', 'ú'=>'u', 'û'=>'u', 'ý'=>'y', 'ý'=>'y', 'þ'=>'b', 'ÿ'=>'y' );
	$bookPart = strtr( $bookPart, $unwanted_array );
	$bookNumber = isset( $truth_bookHash[ $bookPart ] ) ? $truth_bookHash[ $bookPart ] : NULL;
	$chapterNumber = (int) trim( substr( $firstSplit[0], $idx, strlen( $firstSplit[0] ) - $idx ) );
	$verseNumber = 0;
	$endChapterNumber = 0;
	$endVerseNumber = 0;

	$verseSplit = explode( '-', $firstSplit[1] );
	$verseNumber = $verseSplit[0];
	if ( count( $verseSplit ) > 1 ) {
		$endSplit = explode( ':', $verseSplit[1] );
		if (count( $endSplit ) == 1 ) {
		  $endVerseNumber = $endSplit[0];
		} else {
		  $endChapterNumber = (int) $endSplit[0];
		  $endVerseNumber = (int) $endSplit[1];
		}
	}

	return array( $bookNumber, $chapterNumber, $verseNumber, $endChapterNumber, $endVerseNumber );
}

/**************************************
* ASSEMBLE URL FOR VERSES
**************************************/

function get_verse_url( $bookNumber, $chapterNumber, $verseNumber, $endChapterNumber, $endVerseNumber, $bibleVersionID = '' )
{
	if ( 0 == $bookNumber ) return '';
	global $truth_sites, $truth_options;

	$website = 'YouVersion.com';
	$hash = $truth_sites[ $website ];
	$bookName = str_replace( $hash['bookFind'], $hash['bookReplace'], $hash['books'][ $bookNumber ] );
	$result='';

	if ( $endVerseNumber > 0 ) {
		$result = $endChapterNumber > 0 ? $hash['chapterRangeUrl'] : $hash['verseRangeUrl'];
	} elseif ( $verseNumber > 0 ) {
		$result = $hash['singleVerseUrl'];
	} else {
		$result = $hash['singleChapterUrl'];
	}

	if ( '' != $bibleVersionID  ) {
		$bibleVersion = truth_get_bible_version( $bibleVersionID );
	} else {
		! isset( $truth_options['bible_version'] ) ? $truth_options['bible_version'] = '59' : FALSE;
		$bibleVersion = truth_get_bible_version( $truth_options['bible_version'] );
	}

	return str_replace( array( '[book]', '[chapter]', '[verse]', '[endchapter]', '[endverse]', '[version]', '[id]' ), array( $bookName, $chapterNumber, $verseNumber, $endChapterNumber, $endVerseNumber, $bibleVersion['abbr'], $bibleVersion['id'] ), $result );
	break;
}

/**************************************
* ENGINE TO POPULATE THE BOOK HASH
**************************************/

function add_book_to_hash( $number, $names )
{
	global $truth_bookHash;
	foreach ( $names as &$name )
	{
		$truth_bookHash[ $name ] = $number;
	}
}

/**************************************
* POPULATE THE BOOK HASH
**************************************/

function populate_book_hash()
{
	global $truth_bookHash;
	if ( count( $truth_bookHash ) > 0 ) return;

	add_book_to_hash(1, array('genesis','gen','ge','gn'));
	add_book_to_hash(2, array('exodus','exo','ex','exod','exodo'));
	add_book_to_hash(3, array('leviticus','lev','le','lv','levitico'));
	add_book_to_hash(4, array('numbers','num', 'nu', 'nm', 'nb'));
	add_book_to_hash(5, array('deuteronomy','deut','dt','deuteronomio'));
	add_book_to_hash(6, array('joshua','josh','jos','jsh','josue'));
	add_book_to_hash(7, array('judges','judg','jdg','jg','jdgs','jueces','jue','jc'));
	add_book_to_hash(8, array('ruth','rth','ru'));
	add_book_to_hash(9, array('1 samuel','1 sam','1 sa','1samuel','1s','i sa','1 sm','1sa','i sam','1sam','i samuel','1st samuel','first samuel'));
	add_book_to_hash(10, array('2 samuel','2 sam','2 sa','2s','ii sa','2 sm','2sa','ii sam','2sam','ii samuel','2samuel','2nd samuel','second samuel'));
	add_book_to_hash(11, array('1 kings','1 kgs','1 ki','1k','i kgs','1kgs','i ki','1ki','i kings','1kings','1st kgs','1st kings','first kings','first kgs','1kin','1 reyes','1 rey','1 re'));
	add_book_to_hash(12, array('2 kings','2 kgs','2 ki','2k','ii kgs','2kgs','ii ki','2ki','ii kings','2kings','2nd kgs','2nd kings','second kings','second kgs','2kin','2 reyes','2 rey','2 re'));
	add_book_to_hash(13, array('1 chronicles','1 chron','1 ch','i ch','1ch','1 chr','i chr','1chr','i chron','1chron','i chronicles','1chronicles','1st chronicles','first chronicles','1 cronicas','1 cro','1cr'));
	add_book_to_hash(14, array('2 chronicles','2 chron','2 ch','ii ch','2ch','ii chr','2chr','ii chron','2chron','ii chronicles','2chronicles','2nd chronicles','second chronicles','2 cronicas','2 cro','2cr'));
	add_book_to_hash(15, array('ezra','ezr','esdras','esd','es'));
	add_book_to_hash(16, array('nehemiah','neh','ne','nehemias'));
	add_book_to_hash(17, array('esther','esth','es','ester','est'));
	add_book_to_hash(18, array('job','jb'));
	add_book_to_hash(19, array('psalms','pslm','ps','psalm','psa','psm','pss','salmos','sal'));
	add_book_to_hash(20, array('proverbs','prov','pr','prv','pro','proverbios'));
	add_book_to_hash(21, array('ecclesiastes','eccles','ec','ecc','eclesiastes','ecl'));
	add_book_to_hash(22, array('song of solomon','song','so','song of songs','sos','son','cantar de los cantares','cant'));
	add_book_to_hash(23, array('isaiah','isa','is','isaias'));
	add_book_to_hash(24, array('jeremiah','jer','je','jr','jeremias'));
	add_book_to_hash(25, array('lamentations','lam','la','lamentaciones'));
	add_book_to_hash(26, array('ezekiel','ezek','eze','ezk','ezequiel','ezeq','ez'));
	add_book_to_hash(27, array('daniel','dan','da','dn'));
	add_book_to_hash(28, array('hosea','hos','ho','oseas','os'));
	add_book_to_hash(29, array('joel','joe','jl'));
	add_book_to_hash(30, array('amos','am'));
	add_book_to_hash(31, array('obadiah','obad','ob','abdias','abd'));
	add_book_to_hash(32, array('jonah','jnh','jon','jonas'));
	add_book_to_hash(33, array('micah','mic','miqueas','miq'));
	add_book_to_hash(34, array('nahum','nah','na','nahun'));
	add_book_to_hash(35, array('habakkuk','hab','habacuc'));
	add_book_to_hash(36, array('zephaniah','zeph','zep','zp','sofonias','sof'));
	add_book_to_hash(37, array('haggai','hag','hg','ageo','ag'));
	add_book_to_hash(38, array('zechariah','zech','zec','zc','zacarias','zac','za'));
	add_book_to_hash(39, array('malachi','mal','ml','malaquias'));
	add_book_to_hash(40, array('matthew','matt','mt','mat','mateo'));
	add_book_to_hash(41, array('mark','mrk','mk','mr','marcos','mc'));
	add_book_to_hash(42, array('luke','luk','lk','lucas','luc','lc'));
	add_book_to_hash(43, array('john','jn','jhn','juan'));
	add_book_to_hash(44, array('acts','ac','hechos de los apostoles','hech','hch'));
	add_book_to_hash(45, array('romans','rom','ro','rm','romanos'));
	add_book_to_hash(46, array('1 corinthians','1 cor','1 co','i co','1co','i cor','1cor','i corinthians','1corinthians','1st corinthians','first corinthians','1 corin','1 corintios'));
	add_book_to_hash(47, array('2 corinthians','2 cor','2 co','ii co','2co','ii cor','2cor','ii corinthians','2corinthians','2nd corinthians','second corinthians','2 corin','2 corintios'));
	add_book_to_hash(48, array('galatians','gal','ga','galatas'));
	add_book_to_hash(49, array('ephesians','ephes','eph','efesios','efes','ef'));
	add_book_to_hash(50, array('philippians','phil','php','filipenses','fil','flp'));
	add_book_to_hash(51, array('colossians','col','colosenses'));
	add_book_to_hash(52, array('1 thessalonians','1 thess','1 th','i th','1th','i thes','1thes','i thess','1thess','i thessalonians','1thessalonians','1st thessalonians','first thessalonians','1ts','1 tesalonicenses'));
	add_book_to_hash(53, array('2 thessalonians','2 thess','2 th','ii th','2th','ii thes','2thes','ii thess','2thess','ii thessalonians','2thessalonians','2nd thessalonians','second thessalonians','2ts','2 tesalonicenses'));
	add_book_to_hash(54, array('1 timothy','1 tim','1 ti','i ti','1ti','i tim','1tim','i timothy','1timothy','1st timothy','first timothy','1 timoteo'));
	add_book_to_hash(55, array('2 timothy','2 tim','2 ti','ii ti','2ti','ii tim','2tim','ii timothy','2timothy','2nd timothy','second timothy','2 timoteo'));
	add_book_to_hash(56, array('titus','tit','tito'));
	add_book_to_hash(57, array('philemon','philem','phm','filemon','filem','flm'));
	add_book_to_hash(58, array('hebrews','heb','hebreos'));
	add_book_to_hash(59, array('james','jas','jm','santiago','sant','stgo','st'));
	add_book_to_hash(60, array('1 peter','1 pet','1 pe','i pe','1pe','i pet','1pet','i pt','1 pt','1pt','i peter','1peter','1st peter','first peter','1 pedro'));
	add_book_to_hash(61, array('2 peter','2 pet','2 pe','ii pe','2pe','ii pet','2pet','ii pt','2 pt','2pt','ii peter','2peter','2nd peter','second peter','2 pedro'));
	add_book_to_hash(62, array('1 john','1 jn','i jn','1jn','i jo','1jo','i joh','1joh','i jhn','1 jhn','1jhn','i john','1john','1st john','first john','1 juan'));
	add_book_to_hash(63, array('2 john','2 jn','ii jn','2jn','ii jo','2jo','ii joh','2joh','ii jhn','2 jhn','2jhn','ii john','2john','2nd john','second john','2 juan'));
	add_book_to_hash(64, array('3 john','3 jn','iii jn','3jn','iii jo','3jo','iii joh','3joh','iii jhn','3 jhn','3jhn','iii john','3john','3rd john','third john','3 juan'));
	add_book_to_hash(65, array('jude','jud','judas','jds'));
	add_book_to_hash(66, array('revelation','rev','re','apocalipsis','apocalipsis (de juan)','apoc','ap'));
}

/*
	The parameters are:
		singleVerseUrl - The url pattern for looking up a single verse (ex Genesis 1:1)
		verseRangeUrl - The url pattern for looking up a multiverse passage (ex Genesis 1:1-2)
		chapterRangeUrl - The url pattern for looking up a passage that spans multiple chapters (ex Genesis 1:31-2:1)
		bookFind - An array of terms to match in the book name and replace
		bookReplace - An array of terms to replace the matches from bookFind with.  (ex. Use "_" and "-" to convert "1_john" to "1-john")

	You can use the following variables within the url patterns: 
		[book], [chapter], [verse], [endchapter], [endverse]
*/

/**************************************
* SET SITES DATA
**************************************/

function init_truth_sites() {
	//global $truth_options;
	//! isset( $truth_options['bible_version'] ) ? $truth_options['bible_version'] = '59' : FALSE;
	$truth_sites['YouVersion.com'] = array(
		'singleVerseUrl'  => 'http://bible.us/[id]/[book].[chapter].[verse].[version]',
		'verseRangeUrl'   => 'http://bible.us/[id]/[book].[chapter].[verse]-[endverse].[version]',
		'singleChapterUrl'=> 'http://bible.us/[id]/[book].[chapter].[version]',
		'chapterRangeUrl' => '',
		'bookFind'        => array( 'song_of_solomon', ' ' ),
		'bookReplace'     => array( 'song', ''),
		'books'           => array( '', 'gen', 'exo', 'lev', 'num', 'deu', 'jos', 'jdg', 'rut', '1sa', '2sa', '1ki', '2ki', '1ch', '2ch', 'ezr', 'neh', 'est', 'job', 'psa', 'pro', 'ecc', 'sng', 'isa', 'jer', 'lam', 'ezk', 'dan', 'hos', 'jol', 'amo', 'oba', 'jon', 'mic', 'nam', 'hab', 'zep', 'hag', 'zec', 'mal', 'mat', 'mrk', 'luk', 'jhn', 'act', 'rom', '1co', '2co', 'gal', 'eph', 'php', 'col', '1th', '2th', '1ti', '2ti', 'tit', 'phm', 'heb', 'jas', '1pe', '2pe', '1jn', '2jn', '3jn', 'jud', 'rev' )
		);
	return $truth_sites;
}

/**************************************
* SET AVAILABLE VERSIONS
**************************************/

function truth_define_bible_versions() {
	$truth_bible_versions = array(
		'English' => array(
			'8' => array( 'name' => 'Amplified Bible', 'abbr' => 'amp', 'id' => '8' ),
			'12' => array( 'name' => 'American Standard Version', 'abbr' => 'asv', 'id' => '12' ),
			'37' => array( 'name' => 'Common English Bible', 'abbr' => 'ceb', 'id' => '37' ),
			'392' => array( 'name' => 'Contemporary English Version', 'abbr' => 'cevus06', 'id' => '392' ),
			'42' => array( 'name' => 'Catholic Public Domain Version', 'abbr' => 'cpdv', 'id' => '42' ),
			'478' => array( 'name' => 'Darby Translation 1890', 'abbr' => 'darby', 'id' => '478' ),
			'55' => array( 'name' => 'Douay Rheims', 'abbr' => 'dra', 'id' => '55' ),
			'59' => array( 'name' => 'English Standard Version', 'abbr' => 'esv', 'id' => '59' ),
			'416' => array( 'name' => 'Good News Bible', 'abbr' => 'gnbdc', 'id' => '416' ),
			'70' => array( 'name' => 'GOD\'S WORD Translation', 'abbr' => 'gwt', 'id' => '70' ),
			'68' => array( 'name' => 'Good News Translation', 'abbr' => 'gnt', 'id' => '68' ),
			'72' => array( 'name' => 'Holman Christian Standard Bible', 'abbr' => 'hcsb', 'id' => '72' ),
			'1' => array( 'name' => 'King James Version', 'abbr' => 'kjv', 'id' => '1' ),
			'97' => array( 'name' => 'The Message', 'abbr' => 'msg', 'id' => '97' ),
			'100' => array( 'name' => 'New American Standard Bible', 'abbr' => 'nasb', 'id' => '100' ),
			'105' => array( 'name' => 'New Century Version', 'abbr' => 'ncv', 'id' => '105' ),
			'107' => array( 'name' => 'New English Translation', 'abbr' => 'net', 'id' => '107' ),
			'110' => array( 'name' => 'New International Reader\'s Version', 'abbr' => 'nirv', 'id' => '110' ),
			'111' => array( 'name' => 'New International Version', 'abbr' => 'niv', 'id' => '111' ),
			'114' => array( 'name' => 'New King James Version', 'abbr' => 'nkjv', 'id' => '114' ),
			'116' => array( 'name' => 'New Living Translation', 'abbr' => 'nlt', 'id' => '116' ),
			'130' => array( 'name' => 'Orthodox Jewish Bible', 'abbr' => 'ojb', 'id' => '130' ),
			'314' => array( 'name' => 'Tree of Life Bible', 'abbr' => 'tlv', 'id' => '314' ),
			'206' => array( 'name' => 'World English Bible', 'abbr' => 'web', 'id' => '206' ),
			),
		'Español' => array(
			'28' => array( 'name' => 'La Palabra', 'abbr' => 'blph', 'id' => '28'),
			'89' => array( 'name' => 'La Bilia de las Americas', 'abbr' => 'lbla', 'id' => '89' ),
			'128' => array( 'name' => 'Nueva Versión Internacional', 'abbr' => 'nvi', 'id' => '128' ),
			'149' => array( 'name' => 'Biblia Reina Valera 1960', 'abbr' => 'rvr60', 'id' => '149' ),
			),
		);
	return $truth_bible_versions;
}

/**************************************
* GET ARRAY FOR SELECTED BIBLE ID
**************************************/

function truth_get_bible_version( $getID ) {
	global $truth_bible_versions;

	$truth_get_bible_version = array();

	foreach ( $truth_bible_versions as $languageGroup => $languageVersions ) {
		foreach ( $languageVersions as $versionID => $versionInfo ) {
			$getID == $versionID ? $truth_get_bible_version = $versionInfo : FALSE;
		}
	}
	return $truth_get_bible_version;
}

/**************************************
* GET ARRAY FOR SELECTED BIBLE ABBR
**************************************/

function truth_get_bible_version_from_abbr( $abbr ) {
	global $truth_bible_versions;

	$truth_get_bible_version = array();

	foreach ( $truth_bible_versions as $languageGroup => $languageVersions ) {
		foreach ( $languageVersions as $versionID => $versionInfo ) {
			$abbr == $versionInfo['abbr'] ? $truth_get_bible_version = $versionInfo : FALSE;
		}
	}
	return $truth_get_bible_version;
}

/**************************************
* REGISTER AND DEFINE THE SETTINGS
**************************************/
add_action('admin_init', 'truth_admin_init');
function truth_admin_init() {
	register_setting( 'reading', 'truth_settings' );
	add_settings_field( 'bible_version', 'Biblical References', 'truth_bible_version_input', 'reading', 'default' );
}

/**************************************
* DISPLAY VERSION OPTION ON READING SETTINGS PAGE
**************************************/

function truth_bible_version_input() {
	global $truth_options, $truth_bible_versions;

	$truth_notices = new WP_Side_Notice( 'truth' );
	$truth_notices->display();

	ob_start(); ?>

		<p class="description">Used for YouVersion links generated by the <code>Truth</code> plugin.</p>
		<p><label>Default Version: <select id="bible_version" name="truth_settings[bible_version]">
			<?php foreach( $truth_bible_versions as $languageGroup => $languageVersions ): ?>
				<optgroup label="<?php echo $languageGroup?>">
					<?php foreach( $languageVersions as $versionID => $versionInfo ): ?>
						<option value="<?php echo $versionID; ?>" <?php selected( $truth_options['bible_version'], $versionID ); ?>><?php echo $versionInfo['name'] . ' (' . strtoupper( $versionInfo['abbr'] ). ')'; ?></option> <?php
					endforeach; ?>
				</optgroup>
			<?php endforeach; ?>
		</select></label></p>
		<p><input type="checkbox" id="disable_auto_links" name="truth_settings[disable_auto_links]" value="1" <?php checked( $truth_options['disable_auto_links'], 1 ); ?>> Disable auto-generation of links.</input> <span class="description">(maintains use of [truth] shortcode)</span></p>
		<p><label> Append Version to Shortcode Text: <select id="append_version" name="truth_settings[append_version]">
			<option value="none" <?php selected( $truth_options['append_version'], 'none' ); ?>>No (Disabled)</option>
			<option value="abbr" <?php selected( $truth_options['append_version'], 'abbr' ); ?>>Abbreviation</option>
			<option value="full" <?php selected( $truth_options['append_version'], 'full' ); ?>>Full Name</option>
		</select>
		</p>
		<p><input type="checkbox" id="link_in_new_tab" name="truth_settings[link_in_new_tab]" value="1" <?php checked( $truth_options['link_in_new_tab'], 1 ); ?>> Open links in new tab.</input></p>
		<input type="hidden" id="authorize_external_links" name="truth_settings[authorize_external_links]" value="<?php echo $truth_options['authorize_external_links']; ?>"></input>
		
		<?php
	echo ob_get_clean();
}

/**************************************
* SET OPTION REQUIRING AUTHORIZATION
**************************************/

function truth_install() {
	global $truth_options;
	! isset( $truth_options['authorize_external_links'] ) ? $truth_options['authorize_external_links'] = '0' : FALSE;
	! isset( $truth_options['append_version'] ) ? $truth_options['append_version'] = 'full' : FALSE;
	update_option( 'truth_settings', $truth_options );
}
register_activation_hook( __FILE__, 'truth_install' );

/**************************************
* CALL APPROPRIATE NOTICES
**************************************/

function truth_notices() {
	global $pagenow, $truth_options;

	if ( ! isset( $_GET['truth_action'] ) ) {
		$message = 'The <code>Truth</code> plugin needs authorization to link scripture references to <a href="http://youversion.com">YouVersion.com</a>. Click <a href="' . self_admin_url( $pagenow . '?truth_action=permit' ) . '">here</a> to permit this action.';
		truth_showNotice( $message, 'update-nag' );
	} elseif ( isset( $_GET['truth_action'] ) ) {
		if ( 'permit' == $_GET['truth_action'] ) {
			$truth_options['authorize_external_links'] = '1';
			update_option( 'truth_settings', $truth_options );
			$message = 'You have successfully enabled the <code>Truth</code> plugin. You can set a Bible version in the <a href="' . self_admin_url( 'options-reading.php' ) . '"><code>Reading Settings</code></a>.';
			truth_showNotice( $message );
		}
	}
}

/**************************************
* ASSEMBLE NOTICES
**************************************/

function truth_showNotice( $message, $class = 'updated fade' )
{
	if ( 'error' == $class )
		_e( '<div id="truthauth" class="error"><p><strong>' . $message . '</p></div>', 'truth_domain' );
	elseif ( 'update-nag' == $class )
		_e( '<div id="truthauth" class="update-nag">' . $message . '</div>', 'truth_domain' );
	else
		_e( '<div id="truthauth" class="updated fade"><p>' . $message . '</p></div>', 'truth_domain' );
}

function _tdump( $var = '' ) {
    echo '<br /><pre>';
    var_dump( $var ) . '<br />';
    echo '</pre>';
}